Quat

Type Alias Quat 

pub type Quat<T> = Quaternion<T, Unaligned>;
A quaternion representing a rotation.

This quaternion is intended to be normalized, but may denormalize due to floating point “error creep” which can occur when successive quaternion operations are applied.

No SIMD alignment

Quat<T> does not have SIMD alignment, for that use QuatA<T>.

Fields

  • x: T (rotates +Y to +Z)
  • y: T (rotates +Z to +X)
  • z: T (rotates +X to +Y)
  • w: T (the scalar part)

Note that these fields are only exposed by implementing Deref and DerefMut.

Aliased Type

pub struct Quat<T>(/* private fields */);