pub type Affine2A<T> = Affine<2, T, Aligned>;Expand description
A 2D affine transform which can represent translation, rotation, scaling and shear.
Contains a 2x2 matrix and a 2D translation vector.
Prefer using Affine2A<T> over Mat3A<T> for affine transformations,
because it takes less memory and performs better for select operations (see
benchmark results).
SIMD alignment
For appropriate T types, Affine2A<T> has SIMD alignment. For no SIMD
use Affine2<T>.
Aliased Type
#[repr(C)]pub struct Affine2A<T> {
pub submatrix: Matrix<2, T, Aligned>,
pub translation: Vector<2, T, Aligned>,
}Fields
submatrix: Matrix<2, T, Aligned>The part representing rotation, scaling and shear.
translation: Vector<2, T, Aligned>The part representing translation.