pub type Quat<T> = Quaternion<T, Unaligned>;Expand description
A quaternion representing a rotation.
This quaternion is intended to be normalized, but may denormalize due to floating point “error creep” which can occur when successive quaternion operations are applied.
No SIMD alignment
Quat<T> does not have SIMD alignment, for that use QuatA<T>.
Fields
x: T(rotates+Yto+Z)y: T(rotates+Zto+X)z: T(rotates+Xto+Y)w: T(the scalar part)
Note that these fields are only exposed by implementing Deref and
DerefMut.
Aliased Type
pub struct Quat<T>(/* private fields */);