Quat

Type Alias Quat 

pub type Quat<T> = Quaternion<T, Aligned>;
A quaternion representing an orientation.

This quaternion is intended to be of unit length but may denormalize due to floating point “error creep” which can occur when successive quaternion operations are applied.

SIMD Alignment

This type may be SIMD-aligned depending on the target architecture.

Aliased Type

pub struct Quat<T>(/* private fields */);